/**

Game Develop - Path Automatism Extension
Copyright (c) 2010-2011 Florian Rival (Florian.Rival@gmail.com)

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
    claim that you wrote the original software. If you use this software
    in a product, an acknowledgment in the product documentation would be
    appreciated but is not required.

    2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as being the original software.

    3. This notice may not be removed or altered from any source
    distribution.

*/

#include "GDL/ExtensionBase.h"
#include "GDL/Version.h"
#include "PathAutomatism.h"
#include <boost/version.hpp>

/**
 * This class declare information about the extension.
 */
class Extension : public ExtensionBase
{
    public:

        /**
         * Constructor of an extension declares everything the extension contains : Objects, actions, conditions and expressions.
         */
        Extension()
        {
            DECLARE_THE_EXTENSION("PathAutomatism",
                                  _("Automatisme Chemin"),
                                  _("Automatisme permettant de dplacer les objets sur un chemin prdfini"),
                                  "Compil Games",
                                  "zlib/libpng License ( Open Source )")

                DECLARE_AUTOMATISM("PathAutomatism",
                          _("Chemin"),
                          _("Path"),
                          _("Automatisme permettant de dplacer les objets sur un chemin prdfini."),
                          "",
                          "res/path32.png",
                          PathAutomatism,
                          ScenePathDatas)

                    DECLARE_AUTOMATISM_ACTION("SetReverseAtEnd",
                                   _("D/Activer les allers-retours"),
                                   _("Active ou dsactive les aller retours"),
                                   _("Activer les allers-retours de _PARAM0_ : _PARAM2_"),
                                   _("Options"),
                                   "res/actions/window24.png",
                                   "res/actions/window.png",
                                   &PathAutomatism::ActReverseAtEnd);

                        DECLARE_PARAMETER("object", _("Objet"), true, "")
                        DECLARE_PARAMETER("automatism", _("Automatisme"), false, "PhysicsAutomatism")
                        DECLARE_PARAMETER("yesorno", _("Activer"), false, "")

                    DECLARE_END_AUTOMATISM_ACTION()

                    DECLARE_AUTOMATISM_ACTION("SetStopAtEnd",
                                   _("D/Activer l'arrt  la fin du chemin"),
                                   _("Active ou dsactive l'arrt de l'objet  la fin du chemin."),
                                   _("Activer l'rret de _PARAM0_  la fin du chemin : _PARAM2_"),
                                   _("Options"),
                                   "res/actions/window24.png",
                                   "res/actions/window.png",
                                   &PathAutomatism::ActStopAtEnd);

                        DECLARE_PARAMETER("object", _("Objet"), true, "")
                        DECLARE_PARAMETER("automatism", _("Automatisme"), false, "PhysicsAutomatism")
                        DECLARE_PARAMETER("yesorno", _("Activer"), false, "")

                    DECLARE_END_AUTOMATISM_ACTION()

                    DECLARE_AUTOMATISM_CONDITION("ReverseAtEnd",
                                   _("Allers-retours"),
                                   _("Renvoie vrai si l'objet effectue des allers-retours"),
                                   _("_PARAM0_ effectue des allers retours"),
                                   _("Options"),
                                   "res/actions/window24.png",
                                   "res/actions/window.png",
                                   &PathAutomatism::CondReverseAtEnd);

                        DECLARE_PARAMETER("object", _("Objet"), true, "")
                        DECLARE_PARAMETER("automatism", _("Automatisme"), false, "PhysicsAutomatism")
                    DECLARE_END_AUTOMATISM_CONDITION()

                    DECLARE_AUTOMATISM_CONDITION("StopAtEnd",
                                   _("Arrt  la fin du chemin"),
                                   _("Renvoie vrai si l'objet s'arrte  la fin de son chemin."),
                                   _("_PARAM0_ s'arrte  la fin du chemin"),
                                   _("Options"),
                                   "res/actions/window24.png",
                                   "res/actions/window.png",
                                   &PathAutomatism::CondStopAtEnd);

                        DECLARE_PARAMETER("object", _("Objet"), true, "")
                        DECLARE_PARAMETER("automatism", _("Automatisme"), false, "PhysicsAutomatism")

                    DECLARE_END_AUTOMATISM_CONDITION()

                DECLARE_END_AUTOMATISM();

            CompleteCompilationInformation();
        };
        virtual ~Extension() {};

    private:

        /**
         * This function is called by Game Develop so
         * as to complete information about how the extension was compiled ( which libs... )
         * -- Do not need to be modified. --
         */
        void CompleteCompilationInformation()
        {
            #if defined(GD_IDE_ONLY)
            compilationInfo.runtimeOnly = false;
            #else
            compilationInfo.runtimeOnly = true;
            #endif

            #if defined(__GNUC__)
            compilationInfo.gccMajorVersion = __GNUC__;
            compilationInfo.gccMinorVersion = __GNUC_MINOR__;
            compilationInfo.gccPatchLevel = __GNUC_PATCHLEVEL__;
            #endif

            compilationInfo.boostVersion = BOOST_VERSION;

            compilationInfo.sfmlMajorVersion = 2;
            compilationInfo.sfmlMinorVersion = 0;

            #if defined(GD_IDE_ONLY)
            compilationInfo.wxWidgetsMajorVersion = wxMAJOR_VERSION;
            compilationInfo.wxWidgetsMinorVersion = wxMINOR_VERSION;
            compilationInfo.wxWidgetsReleaseNumber = wxRELEASE_NUMBER;
            compilationInfo.wxWidgetsSubReleaseNumber = wxSUBRELEASE_NUMBER;
            #endif

            compilationInfo.gdlVersion = RC_FILEVERSION_STRING;
            compilationInfo.sizeOfpInt = sizeof(int*);

            compilationInfo.informationCompleted = true;
        }
};

/**
 * Used by Game Develop to create the extension class
 * -- Do not need to be modified. --
 */
extern "C" ExtensionBase * GD_EXTENSION_API CreateGDExtension() {
    return new Extension;
}

/**
 * Used by Game Develop to destroy the extension class
 * -- Do not need to be modified. --
 */
extern "C" void GD_EXTENSION_API DestroyGDExtension(ExtensionBase * p) {
    delete p;
}
